# -*- coding: utf-8 -*-
# Copyright (c) 2014-2015, Philip Xu <pyx@xrefactor.com>
# License: BSD New, see LICENSE for details.

from hymn.types.monadplus import MonadPlus
from hymn.types.monoid import Monoid

from hymn.types.continuation import Continuation
from hymn.types.either import Either
from hymn.types.identity import Identity
from hymn.types.list import List
from hymn.types.maybe import Maybe
from hymn.types.reader import Reader
from hymn.types.state import State
from hymn.types.writer import (
    ComplexWriter,
    DecimalWriter,
    FloatWriter,
    FractionWriter,
    ListWriter,
    IntWriter,
    StringWriter,
    TupleWriter,
)


def extract(m):
    return m >> (lambda v: v)


def run_cont(c):
    return c.run()


def run_list(l):
    return list(l)


def run_reader(r):
    return r.run(None)


def run_state(s):
    return s.run(None)


def run_writer(w):
    return w.run()[0]

monad_runners = [
    (Continuation, run_cont),
    (Either, extract),
    (Identity, extract),
    (List, run_list),
    (Maybe, extract),
    (Reader, run_reader),
    (State, run_state),
    (ComplexWriter, run_writer),
    (DecimalWriter, run_writer),
    (FloatWriter, run_writer),
    (FractionWriter, run_writer),
    (ListWriter, run_writer),
    (IntWriter, run_writer),
    (StringWriter, run_writer),
    (TupleWriter, run_writer),
]

monads = [monad for (monad, runner) in monad_runners]
monad_ids = [monad.__name__ for monad in monads]

monadplus_runners = [
    (monad, runner)
    for (monad, runner) in monad_runners
    if issubclass(monad, MonadPlus)
]

monadplus = [monadplus for (monadplus, runner) in monadplus_runners]
monadplus_ids = [monad.__name__ for monad in monadplus]

monoid_runners = [
    (monad, runner)
    for (monad, runner) in monad_runners
    if issubclass(monad, Monoid)
]

monoids = [monoid for (monoid, runner) in monoid_runners]
monoid_ids = [monoid.__name__ for monoid in monoids]

params = [
    ('monad', monads, monad_ids),
    ('monad_runner', monad_runners, monad_ids),
    ('monadplus', monadplus, monadplus_ids),
    ('monadplus_runner', monadplus_runners, monadplus_ids),
    ('monoid', monoids, monoid_ids),
    ('monoid_runner', monoid_runners, monoid_ids),
]


def pytest_generate_tests(metafunc):
    for name, args, ids in params:
        if name in metafunc.funcargnames:
            metafunc.parametrize(name, args, ids=ids)



# -*- coding: utf-8 -*-
import sys
from os import path
from distutils.core import setup

if sys.version_info < (2, 7):
    sys.exit('hymn requires Python 2.7 or higher')

ROOT_DIR = path.abspath(path.dirname(__file__))
sys.path.insert(0, ROOT_DIR)

from hymn import VERSION
from hymn import __doc__ as DESCRIPTION
LONG_DESCRIPTION = open(path.join(ROOT_DIR, 'README.rst')).read()

DEP = [
    'hy>=0.11.0',
]

EXTRA_DEP = {
    'dev': [
        'twine',
    ],
    'doc': [
        'Sphinx>=1.2.3',
    ],
    'test': [
        'pytest>=2.6.4',
    ],
}

HYSRC = ['**.hy']

setup(
    name='hymn',
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    author='Philip Xu',
    author_email='pyx@xrefactor.com',
    url='https://github.com/pyx/hymn/',
    download_url=(
        'https://bitbucket.org/pyx/hymn/get/%s.tar.bz2' % VERSION),
    install_requires=DEP,
    extras_require=EXTRA_DEP,
    packages=['hymn', 'hymn.types'],
    package_data={'hymn': HYSRC, 'hymn.types': HYSRC},
    license='BSD-New',
)
